# Redis Database Copy and Verification Script

This script copies data from one Redis database to another and verifies the copy operation. It prompts the user to enter the necessary credentials for both the source and target Redis databases.

## Requirements

- Python 3.x
- Redis server running and accessible
- Required Python packages (see `requirements.txt`)

## Installation

1. **Create a virtual environment** (optional but recommended):
    `python3 -m venv .venv source .venv/bin/activate`

2. **Install the required packages**:
    `pip install -r requirements.txt`

## Configuration

Before running the script, you need to update the connection details for your Redis databases in the script.

1. **Open the script**:
    `nano copy_redis_db.py`

2. **Update the connection details** in the Configuration Section:

    ```python
    # Source Redis database connection details
    SOURCE_HOST = 'your_source_redis_host'
    SOURCE_PORT = your_source_redis_port
    SOURCE_DB = your_source_redis_db

    # Target Redis database connection details
    TARGET_HOST = 'your_target_redis_host'
    TARGET_PORT = your_target_redis_port
    TARGET_DB = your_target_redis_db
    ```

3. **Optionally, modify performance settings**:

    ```python
    BATCH_SIZE = 1000 # Number of keys to process in each batch
    MAX_WORKERS = 8 # Maximum number of workers to use for parallel processing
    ```

## Usage

1. **Run the script**:

    ```python
    python3 copy_redis_db.py
    ```

2. **Enter the required credentials** when prompted:

    ```text
    Please enter the source Redis connection details:
    Source Database Username (optional, required for Redis v. 7 and higher): source_username
    Source Database Password: ********

    Please enter the target Redis connection details:
    Target Database Username (optional, required for Redis v. 7 and higher): target_username
    Target Database Password: ********
    ```

3. **The script will copy the data** from the source Redis database to the target Redis database and verify the copy operation.

## Logging

The script generates a log file with detailed information about the copy and verification processes. The log file is named `redis_verification_<timestamp>.log` and is located in the same directory as the script.

## Error Handling

The script handles various errors that may occur during the connection, copy, and verification processes. If an error occurs, the script will print an error message and log the details in the log file.

## Example

Here is an example of running the script:

```text
copy_redis_db.py

Please enter the source Redis connection details:
Source Database Username (optional, required for Redis v. 7 and higher): source_username
Source Database Password: ********

Please enter the target Redis connection details:
Target Database Username (optional, required for Redis v. 7 and higher): target_username
Target Database Password: ********

Connecting to source database at your_source_redis_host:your_source_redis_port, DB: your_source_redis_db...
Connecting to target database at your_target_redis_host:your_target_redis_port, DB: your_target_redis_db...

Retrieving keys from source database...
Found 1000 keys to copy.

Initiating data copy process...
Data copy completed successfully.
Copy operation finished in 10.00 seconds (100 keys/s).

Verifying keys...
All data was successfully copied and verified.
Data verification completed in 5.00 seconds (200 keys/s).
```

## Troubleshooting

If you encounter any issues, please check the log file for detailed error messages and possible causes. Common issues include:

- Incorrect host, port, username, or password
- Redis server is not running
- Network issues
- SSL settings mismatch

## Acknowledgements

This script uses the `redis` Python package for interacting with Redis databases. Special thanks to the contributors of the `redis` package.
